@@ -31,9 +31,9 @@
 
 PanelAgent::PanelAgent(QObject *parent)
     : QObject(parent)
+    ,adaptor(new ImpanelAdaptor(this))
+    ,adaptor2(new Impanel2Adaptor(this))
 {
-    // constructor
-    adaptor = new ImpanelAdaptor(this);
 
     QDBusConnection::connectToBus(QDBusConnection::SessionBus, "kimpanel_bus").registerObject("/org/kde/impanel", this);
     QDBusConnection::connectToBus(QDBusConnection::SessionBus, "kimpanel_bus").registerService("org.kde.impanel");
@@ -47,6 +47,8 @@ PanelAgent::PanelAgent(QObject *parent)
                                             "ShowAux", this, SIGNAL(showAux(bool)));
     QDBusConnection("kimpanel_bus").connect("", "", "org.kde.kimpanel.inputmethod",
                                             "ShowLookupTable", this, SIGNAL(showLookupTable(bool)));
+    QDBusConnection("kimpanel_bus").connect("", "", "org.kde.kimpanel.inputmethod",
+                                            "UpdateLookupTableCursor", this, SIGNAL(updateLookupTableCursor(int)));
 
     // do some serialization
     QDBusConnection("kimpanel_bus").connect("", "", "org.kde.kimpanel.inputmethod",
@@ -194,6 +196,7 @@ static KimpanelLookupTable Args2LookupTable(const QStringList &labels, const QSt
 void PanelAgent::created()
 {
     emit PanelCreated();
+    emit PanelCreated2();
 }
 
 void PanelAgent::exit()
@@ -262,3 +265,7 @@ void PanelAgent::ExecMenu(const QStringList &entries)
     emit execMenu(list);
 }
 
+void PanelAgent::SetSpotRect(int x, int y, int w, int h)
+{
+    emit updateSpotRect(x, y, w, h);
+}
