@@ -54,9 +54,8 @@ MediaWikiRunner::MediaWikiRunner(QObject *parent, const QVariantList& args)
         m_icon = KIcon(info.icon());
     }
 
-    Plasma::RunnerSyntax s(i18nc("Mediawiki runner syntax", "wiki :q:"),
-                           i18n("Searches for :q: in the MediaWiki at %1.", m_apiUrl.toString()));
-    addSyntax(s);
+
+    addSyntax(Plasma::RunnerSyntax("wiki :q:", i18n("Searches %1 for :q:.", m_name)));
 
     setSpeed( SlowSpeed );
 }
@@ -71,10 +70,12 @@ void MediaWikiRunner::match(Plasma::RunnerContext &context)
     // TODO: check for networkconnection
 
     QString term = context.query();
-    if (!term.startsWith("wiki ")) {
-        return;
-    } else {
-        term = term.remove("wiki ");
+    if (!context.singleRunnerQueryMode()) {
+        if (!term.startsWith("wiki ")) {
+            return;
+        } else {
+            term = term.remove("wiki ");
+        }
     }
     if (!m_apiUrl.isValid() || term.length() < 3) {
         //kDebug() << "yours is too short" << term;
@@ -94,7 +95,11 @@ void MediaWikiRunner::match(Plasma::RunnerContext &context)
     }
 
     MediaWiki mediawiki;
-    mediawiki.setMaxItems(3);
+    if (context.singleRunnerQueryMode()) {
+        mediawiki.setMaxItems(10);
+    } else {
+        mediawiki.setMaxItems(3);
+    }
     mediawiki.setApiUrl( m_apiUrl );
     connect( &mediawiki, SIGNAL(finished(bool)), &loop, SLOT(quit()) );
 
@@ -106,6 +111,9 @@ void MediaWikiRunner::match(Plasma::RunnerContext &context)
     if (!context.isValid()) {
         return;
     }
+    qreal relevance = 0.5;
+    qreal stepRelevance = 0.1;
+
     foreach(const MediaWiki::Result& res, mediawiki.results()) {
         kDebug() << "Match:" << res.url << res.title;
         Plasma::QueryMatch match(this);
@@ -113,6 +121,9 @@ void MediaWikiRunner::match(Plasma::RunnerContext &context)
         match.setIcon(m_icon);
         match.setText(QString("%1: %2").arg(m_name, res.title));
         match.setData(res.url);
+        match.setRelevance(relevance);
+        relevance +=stepRelevance;
+        stepRelevance *=0.5;
         context.addMatch(res.title, match);
     }
 }
